@@ -79,6 +79,35 @@ typedef enum getdns_tls_hs_state {
 	GETDNS_HS_FAILED
 } getdns_tls_hs_state_t;
 
+typedef enum getdns_tsig_algo {
+	GETDNS_NO_TSIG     = 0, /* Do not use tsig */
+	GETDNS_HMAC_MD5    = 1, /* 128 bits */
+	GETDNS_GSS_TSIG    = 2, /* Not supported */
+	GETDNS_HMAC_SHA1   = 3, /* 160 bits */
+	GETDNS_HMAC_SHA224 = 4,
+	GETDNS_HMAC_SHA256 = 5,
+	GETDNS_HMAC_SHA384 = 6,
+	GETDNS_HMAC_SHA512 = 7
+} getdns_tsig_algo;
+
+typedef struct getdns_tsig_info {
+	getdns_tsig_algo  alg;
+	const char       *name;
+	size_t            strlen_name;
+	const uint8_t    *dname;
+	size_t            dname_len;
+	size_t            min_size; /* in # octets */
+	size_t            max_size; /* Actual size in # octets */
+} getdns_tsig_info;
+
+const getdns_tsig_info *_getdns_get_tsig_info(getdns_tsig_algo tsig_alg);
+
+/* for doing public key pinning of TLS-capable upstreams: */
+typedef struct sha256_pin {
+	char pin[SHA256_DIGEST_LENGTH];
+	struct sha256_pin *next;
+} sha256_pin_t;
+
 typedef struct getdns_upstream {
 	/* backpointer to containing upstreams structure */
 	struct getdns_upstreams *upstreams;
@@ -89,6 +118,7 @@ typedef struct getdns_upstream {
 	/* How is this upstream doing? */
 	size_t                   writes_done;
 	size_t                   responses_received;
+	uint64_t                 keepalive_timeout;
 	int                      to_retry;
 	int                      back_off;
 
@@ -97,12 +127,12 @@ typedef struct getdns_upstream {
 	getdns_transport_list_t  transport;
 	SSL*                     tls_obj;
 	getdns_tls_hs_state_t    tls_hs_state;
-	getdns_dns_req *         starttls_req;
 	getdns_eventloop_event   event;
 	getdns_eventloop        *loop;
 	getdns_tcp_state         tcp;
 	char                     tls_auth_name[256];
 	size_t                   tls_auth_failed;
+	sha256_pin_t            *tls_pubkey_pinset;
 
 	/* Pipelining of TCP network requests */
 	getdns_network_req      *write_queue;
@@ -119,6 +149,14 @@ typedef struct getdns_upstream {
 	unsigned has_prev_client_cookie : 1;
 	unsigned has_server_cookie : 1;
 	unsigned server_cookie_len : 5;
+	unsigned tls_fallback_ok : 1;
+
+	/* TSIG */
+	uint8_t          tsig_dname[256];
+	size_t           tsig_dname_len;
+	size_t           tsig_size;
+	uint8_t          tsig_key[256];
+	getdns_tsig_algo tsig_alg;
 
 } getdns_upstream;
 
@@ -138,9 +176,16 @@ struct getdns_context {
 	uint64_t             timeout;
 	uint64_t             idle_timeout;
 	getdns_redirects_t   follow_redirects;
-	struct getdns_list   *dns_root_servers;
+	getdns_list          *dns_root_servers;
+	char                 root_servers_fn[FILENAME_MAX];
 	getdns_append_name_t append_name;
-	struct getdns_list   *suffix;
+	/* Suffix buffer containing a list of (length byte | dname) where 
+	 * length bytes contains the length of the following dname.
+	 * The last dname should be the zero byte.
+	 */
+	const uint8_t        *suffixes;
+	/* Length of all suffixes in the suffix buffer */
+	size_t               suffixes_len; 
 	uint8_t              *trust_anchors;
 	size_t                trust_anchors_len;
 	getdns_upstreams     *upstreams;
@@ -240,8 +285,7 @@ getdns_return_t _getdns_context_cancel_request(struct getdns_context *context,
 char *_getdns_strdup(const struct mem_funcs *mfs, const char *str);
 
 struct getdns_bindata *_getdns_bindata_copy(
-    struct mem_funcs *mfs,
-    const struct getdns_bindata *src);
+    struct mem_funcs *mfs, size_t size, const uint8_t *data);
 
 void _getdns_bindata_destroy(
     struct mem_funcs *mfs,
