@@ -30,6 +30,7 @@
 
 #include <getdns/getdns.h>
 #include <sys/time.h>
+#include <stdio.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -341,7 +342,62 @@ getdns_context_get_update_callback(getdns_context *context, void **userarg,
  */
 const char *getdns_get_errorstr_by_id(uint16_t err);
 
+	
 
+/**
+ * Public Key Pinning functionality:
+ * 
+ * a public key pinset is a list of dicts.  each dict should have a
+ * "digest" and a "value".
+ * 
+ * "digest": a string indicating the type of digest. at the moment, we
+ *           only support a "digest" of "sha256".
+ * 
+ * "value": a binary representation of the digest provided.
+ * 
+ * given a such a pinset, we should be able to validate a chain
+ * properly according to section 2.6 of RFC 7469.
+ */
+
+/**
+ * convert an HPKP-style pin description to an appropriate getdns data
+ * structure.  An example string is: (with the quotes, without any
+ * leading or trailing whitespace):
+ *
+ *    pin-sha256="E9CZ9INDbd+2eRQozYqqbQ2yXLVKB9+xcprMF+44U1g="
+ *
+ * It is the caller's responsibility to call getdns_dict_destroy() on
+ * the dict returned when it is no longer needed.
+ *
+ * @param context a context to use to create the dict, or NULL to create
+ *            it generically
+ * @param str the pinning string to parse
+ * @return a dict created from ctx, or  NULL if the string did not match. 
+ */
+getdns_dict* getdns_pubkey_pin_create_from_string(
+	getdns_context* context,
+	const char* str);
+
+
+/**
+ * Test whether a given pinset is reasonable, including:
+ *
+ * is it well-formed?
+ * are there at least two pins?
+ * are the digests used sane?
+ *
+ * @param pinset the set of public key pins to check for sanity.  This
+ *               should be a list of dicts.
+ * @return errorlist if not NULL, a list of human-readable strings is 
+ *                   appended to errorlist.
+ * @return GETDNS_RETURN_GOOD if the pinset passes the sanity check.
+ */ 
+getdns_return_t getdns_pubkey_pinset_sanity_check(
+	const getdns_list* pinset,
+	getdns_list* errorlist);
+
+
+	
 /* WARNING! Function getdns_strerror is not in the API specification and
  * is likely to be removed from future versions of our implementation, to be
  * replaced by getdns_get_errorstr_by_id or something similar.
@@ -349,10 +405,10 @@ const char *getdns_get_errorstr_by_id(uint16_t err);
  */
 getdns_return_t getdns_strerror(getdns_return_t err, char *buf, size_t buflen);
 
-#define GETDNS_VERSION "0.5.1"
-#define GETDNS_NUMERIC_VERSION 0x00050100
-#define GETDNS_API_VERSION "October 2015"
-#define GETDNS_API_NUMERIC_VERSION 0x07df0a00
+#define GETDNS_VERSION "0.9.0"
+#define GETDNS_NUMERIC_VERSION 0x00090000
+#define GETDNS_API_VERSION "December 2015"
+#define GETDNS_API_NUMERIC_VERSION 0x07df0c00
 
 const char *getdns_get_version(void);
 uint32_t getdns_get_version_number(void);
@@ -362,15 +418,18 @@ uint32_t getdns_get_api_version_number(void);
 /* Authentication options used when doing TLS */
 typedef enum getdns_tls_authentication_t {
 	GETDNS_AUTHENTICATION_NONE = 1300,
-	GETDNS_AUTHENTICATION_HOSTNAME = 1301,
+	GETDNS_AUTHENTICATION_REQUIRED = 1301
 } getdns_tls_authentication_t;
 
+/* an alias for REQUIRED */
+#define GETDNS_AUTHENTICATION_HOSTNAME GETDNS_AUTHENTICATION_REQUIRED
+
 /**
   * \defgroup Base authentication texts
   * @{
   */
 #define GETDNS_AUTHENTICATION_NONE_TEXT "See getdns_context_set_tls_authentication()"
-#define GETDNS_AUTHENTICATION_HOSTNAME_TEXT "See getdns_context_set_tls_authentication()"
+#define GETDNS_AUTHENTICATION_REQUIRED_TEXT "See getdns_context_set_tls_authentication()"
 /** @}
   */
 
@@ -380,6 +439,8 @@ typedef enum getdns_tls_authentication_t {
 #define GETDNS_CONTEXT_CODE_EDNS_CLIENT_SUBNET_PRIVATE_TEXT "Change related to getdns_context_set_edns_client_subnet_private"
 #define GETDNS_CONTEXT_CODE_TLS_QUERY_PADDING_BLOCKSIZE 620
 #define GETDNS_CONTEXT_CODE_TLS_QUERY_PADDING_BLOCKSIZE_TEXT "Change related to getdns_context_set_tls_query_padding_blocksize"
+#define GETDNS_CONTEXT_CODE_PUBKEY_PINSET 621
+#define GETDNS_CONTEXT_CODE_PUBKEY_PINSET_TEXT "Change related to getdns_context_set_pubkey_pinset"
 
 getdns_return_t
 getdns_context_set_tls_authentication(
@@ -399,8 +460,180 @@ getdns_context_get_tls_authentication(getdns_context *context,
 #define GETDNS_TRANSPORT_TLS_ONLY_KEEP_CONNECTIONS_OPEN_TEXT "See getdns_context_set_dns_transport()"
 #define GETDNS_TRANSPORT_TLS_FIRST_AND_FALL_BACK_TO_TCP_KEEP_CONNECTIONS_OPEN 545
 #define GETDNS_TRANSPORT_TLS_FIRST_AND_FALL_BACK_TO_TCP_KEEP_CONNECTIONS_OPEN_TEXT "See getdns_context_set_dns_transport()"
-#define GETDNS_TRANSPORT_STARTTLS_FIRST_AND_FALL_BACK_TO_TCP_KEEP_CONNECTIONS_OPEN 546
-#define GETDNS_TRANSPORT_STARTTLS_FIRST_AND_FALL_BACK_TO_TCP_KEEP_CONNECTIONS_OPEN_TEXT "See getdns_context_set_dns_transport()"
+
+#define GETDNS_RETURN_NEED_MORE_SPACE ((getdns_return_t) 399 )
+#define GETDNS_RETURN_NEED_MORE_SPACE_TEXT "The buffer was too small"
+
+/**
+ * Convert rr_dict to wireformat representation of the resource record.
+ *
+ * @param  rr_dict The getdns dict representation of the resource record
+ * @return wire    A newly allocated buffer which will contain the wireformat.
+ * @return wire_sz The size of the allocated buffer and the wireformat.
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ */
+getdns_return_t
+getdns_rr_dict2wire(
+    const getdns_dict *rr_dict, uint8_t **wire, size_t *wire_sz);
+
+/**
+ * Convert rr_dict to wireformat representation of the resource record.
+ *
+ * @param  rr_dict The getdns dict representation of the resource record
+ * @param  wire    The buffer in which the wireformat will be written
+ * @param  wire_sz On input the size of the wire buffer,
+ *                 On output the amount of wireformat needed for the
+ *                 wireformat representation of the resource record;
+ *                 even if it did not fit.
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ * GETDNS_RETURN_NEED_MORE_SPACE will be returned when the buffer was too
+ * small.  wire_sz will be set to the needed buffer space then.
+ */
+getdns_return_t
+getdns_rr_dict2wire_buf(
+    const getdns_dict *rr_dict, uint8_t *wire, size_t *wire_sz);
+
+/**
+ * Convert rr_dict to wireformat representation of the resource record.
+ *
+ * @param  rr_dict The getdns dict representation of the resource record
+ * @param  wire    A pointer to the buffer pointer in which the wireformat 
+ *                 will be written.
+ *                 On output the buffer pointer will have moved along
+ *                 the buffer and point right after the just written RR.
+ * @param  wire_sz On input the size of the wire buffer,
+ *                 On output the amount of wireformat needed for the
+ *                 wireformat will have been substracted from wire_sz.
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ * GETDNS_RETURN_NEED_MORE_SPACE will be returned when the buffer was too
+ * small.  The function will pretend that it had written beyond the end
+ * of the buffer, and wire will point past the buffer and wire_sz will
+ * contain a negative value.
+ */
+getdns_return_t
+getdns_rr_dict2wire_scan(
+    const getdns_dict *rr_dict, uint8_t **wire, int *wire_sz);
+
+
+/**
+ * Convert wireformat resource record in a getdns rr_dict representation.
+ *
+ * @param  wire    Buffer containing the wireformat rr
+ * @param  wire_sz Size of the wire buffer
+ * @return rr_dict The returned rr_dict
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ */
+getdns_return_t
+getdns_wire2rr_dict(
+    const uint8_t *wire, size_t wire_sz, getdns_dict **rr_dict);
+
+/**
+ * Convert wireformat resource record in a getdns rr_dict representation.
+ *
+ * @param  wire    Buffer containing the wireformat rr
+ * @param  wire_sz On input the size of the wire buffer
+ *                 On output the length of the wireformat rr.
+ * @return rr_dict The returned rr_dict
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ */
+getdns_return_t
+getdns_wire2rr_dict_buf(
+    const uint8_t *wire, size_t *wire_sz, getdns_dict **rr_dict);
+
+/**
+ * Convert wireformat resource record in a getdns rr_dict representation.
+ *
+ * @param  wire    A pointer to the pointer of the wireformat buffer.
+ *                 On return this pointer is moved to after first read
+ *                 in resource record.
+ * @param  wire_sz On input the size of the wire buffer
+ *                 On output the size is decreased with the length
+ *                 of the wireformat resource record.
+ * @return rr_dict The returned rr_dict
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ */
+getdns_return_t
+getdns_wire2rr_dict_scan(
+    const uint8_t **wire, size_t *wire_sz, getdns_dict **rr_dict);
+
+
+/**
+ * Convert rr_dict to the string representation of the resource record.
+ *
+ * @param  rr_dict The getdns dict representation of the resource record
+ * @return str     A newly allocated string representation of the rr
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ */
+getdns_return_t
+getdns_rr_dict2str(
+    const getdns_dict *rr_dict, char **str);
+
+/**
+ * Convert rr_dict to the string representation of the resource record.
+ *
+ * @param  rr_dict The getdns dict representation of the resource record
+ * @param  str     The buffer in which the string will be written
+ * @param  str_len On input the size of the text buffer,
+ *                 On output the amount of characters needed to write
+ *                 the string representation of the rr.  Even if it does
+ *                 not fit.
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ * GETDNS_RETURN_NEED_MORE_SPACE will be returned when the buffer was too
+ * small.  str_len will be set to the needed buffer space then.
+ */
+getdns_return_t
+getdns_rr_dict2str_buf(
+    const getdns_dict *rr_dict, char *str, size_t *str_len);
+
+/**
+ * Convert rr_dict to the string representation of the resource record.
+ *
+ * @param  rr_dict The getdns dict representation of the resource record
+ * @param  str     A pointer to the buffer pointer in which the string 
+ *                 will be written.
+ *                 On output the buffer pointer will have moved along
+ *                 the buffer and point right after the just written RR.
+ * @param  str_len On input the size of the str buffer,
+ *                 On output the number of characters needed for the
+ *                 string will have been substracted from strlen.
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ * GETDNS_RETURN_NEED_MORE_SPACE will be returned when the buffer was too
+ * small.  The function will pretend that it had written beyond the end
+ * of the buffer, and str will point past the buffer and str_len will
+ * contain a negative value.
+ */
+getdns_return_t
+getdns_rr_dict2str_scan(
+    const getdns_dict *rr_dict, char **str, int *str_len);
+
+
+/**
+ * Convert the string representation of the resource record to rr_dict format.
+ *
+ * @param  str        String representation of the resource record.
+ * @return rr_dict     The result getdns dict representation of the resource record
+ * @param  origin      Default suffix for not fully qualified domain names
+ * @param  default_ttl Default ttl
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ */
+getdns_return_t
+getdns_str2rr_dict(
+    const char *str, getdns_dict **rr_dict,
+    const char *origin, uint32_t default_ttl);
+
+/**
+ * Read the zonefile and convert to a list of rr_dict's.
+ *
+ * @param  fp          String representation of the resource record.
+ * @return rr_list     The result list of rr_dicts representing the zone file.
+ * @param  origin      Default suffix for not fully qualified domain names
+ * @param  default_ttl Default ttl
+ * @return GETDNS_RETURN_GOOD on success or an error code on failure.
+ */
+getdns_return_t
+getdns_fp2rr_list(
+    FILE *in, getdns_list **rr_list,
+    const char *origin, uint32_t default_ttl);
 
 #ifdef __cplusplus
 }
