@@ -165,6 +165,7 @@ static gldns_lookup_table gldns_edns_options_data[] = {
 	{ 6, "DHU" },
 	{ 7, "N3U" },
 	{ 8, "edns-client-subnet" },
+	{ 11, "edns-tcp-keepalive"},
 	{ 0, NULL}
 };
 gldns_lookup_table* gldns_edns_options = gldns_edns_options_data;
@@ -1833,6 +1834,25 @@ int gldns_wire2str_edns_subnet_print(char** s, size_t* sl, uint8_t* data,
 	return w;
 }
 
+int gldns_wire2str_edns_keepalive_print(char** s, size_t* sl, uint8_t* data,
+	size_t len)
+{
+	int w = 0;
+	uint16_t timeout;
+	if(!(len == 0 || len == 2)) {
+		w += gldns_str_print(s, sl, "malformed keepalive ");
+		w += print_hex_buf(s, sl, data, len);
+		return w;
+	}
+	if(len == 0 ) {
+		w += gldns_str_print(s, sl, "no timeout value (only valid for client option) ");
+	} else {
+		timeout = gldns_read_uint16(data);
+		w += gldns_str_print(s, sl, "timeout value in units of 100ms %u", (int)timeout);
+	}
+	return w;
+}
+
 int gldns_wire2str_edns_option_print(char** s, size_t* sl,
 	uint16_t option_code, uint8_t* optdata, size_t optlen)
 {
@@ -1861,6 +1881,9 @@ int gldns_wire2str_edns_option_print(char** s, size_t* sl,
 	case GLDNS_EDNS_CLIENT_SUBNET:
 		w += gldns_wire2str_edns_subnet_print(s, sl, optdata, optlen);
 		break;
+	 case GLDNS_EDNS_KEEPALIVE:
+		w += gldns_wire2str_edns_keepalive_print(s, sl, optdata, optlen);
+		break; 
 	default:
 		/* unknown option code */
 		w += print_hex_buf(s, sl, optdata, optlen);
