@@ -107,6 +107,13 @@ _getdns_list_remove_name(getdns_list *list, const char *name)
 
 	i = &list->items[index];
 	if (!*next) {
+		switch (i->dtype) {
+		case t_dict   : getdns_dict_destroy(i->data.dict); break;
+		case t_list   : getdns_list_destroy(i->data.list); break;
+		case t_bindata: _getdns_bindata_destroy(
+		                    &list->mf, i->data.bindata);
+		default       : break;
+		}
 		if (index < list->numinuse - 1)
 			(void) memmove( i, &i[1],
 			    (list->numinuse - index) * sizeof(getdns_item));
@@ -545,17 +552,17 @@ getdns_list_set_list(
 }				/* getdns_list_set_list */
 
 /*---------------------------------------- getdns_list_set_bindata */
-getdns_return_t
-getdns_list_set_bindata(
-    getdns_list *list, size_t index, const getdns_bindata *child_bindata)
+static getdns_return_t
+_getdns_list_set_const_bindata(
+    getdns_list *list, size_t index, size_t size, const void *data)
 {
 	getdns_bindata *newbindata;
 	getdns_return_t r;
 
-	if (!list || !child_bindata)
+	if (!list)
 		return GETDNS_RETURN_INVALID_PARAMETER;
 
-	if (!(newbindata = _getdns_bindata_copy(&list->mf, child_bindata)))
+	if (!(newbindata = _getdns_bindata_copy(&list->mf, size, data)))
 		return GETDNS_RETURN_MEMORY_ERROR;
 
 	if ((r = _getdns_list_request_index(list, index))) {
@@ -567,32 +574,22 @@ getdns_list_set_bindata(
 	return GETDNS_RETURN_GOOD;
 }				/* getdns_list_set_bindata */
 
+getdns_return_t
+getdns_list_set_bindata(
+    getdns_list *list, size_t index, const getdns_bindata *child_bindata)
+{
+	return !child_bindata ? GETDNS_RETURN_INVALID_PARAMETER
+	    : _getdns_list_set_const_bindata(
+	    list, index, child_bindata->size, child_bindata->data);
+}
+
 /*----------------------------------------- getdns_list_set_string */
 static getdns_return_t
 getdns_list_set_string(getdns_list *list, size_t index, const char *value)
 {
-	getdns_bindata *newbindata;
-	getdns_return_t r;
-
-	if (!list || !value)
-		return GETDNS_RETURN_INVALID_PARAMETER;
-
-	if (!(newbindata = GETDNS_MALLOC(list->mf, getdns_bindata)))
-		return GETDNS_RETURN_MEMORY_ERROR;
-
-	newbindata->size = strlen(value);
-	if (!(newbindata->data = (void *)_getdns_strdup(&list->mf, value))) {
-		GETDNS_FREE(list->mf, newbindata);
-		return GETDNS_RETURN_MEMORY_ERROR;
-	}
-	if ((r = _getdns_list_request_index(list, index))) {
-		GETDNS_FREE(list->mf, newbindata->data);
-		GETDNS_FREE(list->mf, newbindata);
-		return r;
-	}
-	list->items[index].dtype = t_bindata;
-	list->items[index].data.bindata = newbindata;
-	return GETDNS_RETURN_GOOD;
+	return value
+	    ? _getdns_list_set_const_bindata(list, index, strlen(value), value)
+	    : GETDNS_RETURN_INVALID_PARAMETER;
 }				/* getdns_list_set_string */
 
 /*---------------------------------------- getdns_list_set_int */
@@ -631,6 +628,13 @@ _getdns_list_append_bindata(getdns_list *list, const getdns_bindata *child_binda
 	return getdns_list_set_bindata(list, list->numinuse, child_bindata);
 }
 getdns_return_t
+_getdns_list_append_const_bindata(
+    getdns_list *list, size_t size, const void *data)
+{
+	if (!list) return GETDNS_RETURN_INVALID_PARAMETER;
+	return _getdns_list_set_const_bindata(list, list->numinuse, size, data);
+}
+getdns_return_t
 _getdns_list_append_string(getdns_list *list, const char *value)
 {
 	if (!list) return GETDNS_RETURN_INVALID_PARAMETER;
