@@ -39,7 +39,7 @@ rr_iter_find_nxt(_getdns_rr_iter *i)
 	assert(i);
 	assert(i->rr_type);
 
-	i->nxt = i->n < GLDNS_QDCOUNT(i->pkt)
+	i->nxt = i->pkt && i->n < GLDNS_QDCOUNT(i->pkt)
 	       ? i->rr_type + 4
 	       : i->rr_type + 10 > i->pkt_end
 	       ? i->pkt_end
@@ -51,13 +51,14 @@ rr_iter_find_nxt(_getdns_rr_iter *i)
 static _getdns_rr_iter *
 find_rrtype(_getdns_rr_iter *i)
 {
-	uint8_t *pos;
+	const uint8_t *pos;
 
 	assert(i);
 	assert(i->pos);
 
 	/* Past the last RR in the pkt */
-	if (GLDNS_QDCOUNT(i->pkt) + GLDNS_ANCOUNT(i->pkt) +
+	if (i->pkt &&
+	    GLDNS_QDCOUNT(i->pkt) + GLDNS_ANCOUNT(i->pkt) +
 	    GLDNS_NSCOUNT(i->pkt) + GLDNS_ARCOUNT(i->pkt) <= i->n)
 		goto done;
 
@@ -83,7 +84,7 @@ done:
 }
 
 _getdns_rr_iter *
-_getdns_rr_iter_init(_getdns_rr_iter *i, uint8_t *pkt, size_t pkt_len)
+_getdns_rr_iter_init(_getdns_rr_iter *i, const uint8_t *pkt, size_t pkt_len)
 {
 	assert(i);
 
@@ -99,6 +100,25 @@ _getdns_rr_iter_init(_getdns_rr_iter *i, uint8_t *pkt, size_t pkt_len)
 	return find_rrtype(i);
 }
 
+_getdns_rr_iter *
+_getdns_single_rr_iter_init(
+    _getdns_rr_iter *i, const uint8_t *wire, size_t wire_len)
+{
+	assert(i);
+
+	if (!wire || wire_len < 5 /* name + type + class */) {
+		i->pos = NULL;
+		return NULL;
+	}
+	i->pkt     = NULL;
+	i->pos     = wire;
+	i->pkt_end = wire + wire_len;
+	i->n       = 0;
+
+	return find_rrtype(i);
+}
+
+
 _getdns_rr_iter *
 _getdns_rr_iter_rewind(_getdns_rr_iter *i)
 {
@@ -121,14 +141,14 @@ _getdns_rr_iter_next(_getdns_rr_iter *i)
 	return find_rrtype(i);
 }
 
-static uint8_t *
-dname_if_or_as_decompressed(uint8_t *pkt, uint8_t *pkt_end, uint8_t *pos,
-    uint8_t *buf, size_t *len, size_t refs)
+static const uint8_t *
+dname_if_or_as_decompressed(const uint8_t *pkt, const uint8_t *pkt_end,
+    const uint8_t *pos, uint8_t *buf, size_t *len, size_t refs)
 {
 	uint16_t offset;
-	uint8_t *start, *dst;
+	const uint8_t *start;
+	uint8_t *dst;
 
-	assert(pkt);
 	assert(pkt_end);
 	assert(pos);
 	assert(buf);
@@ -138,7 +158,7 @@ dname_if_or_as_decompressed(uint8_t *pkt, uint8_t *pkt_end, uint8_t *pos,
 		goto error;
 
 	if ((*pos & 0xC0) == 0xC0) {
-		if (pos + 1 >= pkt_end)
+		if (!pkt || pos + 1 >= pkt_end)
 			goto error;
 		offset = gldns_read_uint16(pos) & 0x3FFF;
 		if (pkt + offset >= pkt_end)
@@ -175,7 +195,7 @@ dname_if_or_as_decompressed(uint8_t *pkt, uint8_t *pkt_end, uint8_t *pos,
 			start = pos;
 		}
 		if ((*pos & 0xC0) == 0xC0) {
-			if (pos + 1 >= pkt_end)
+			if (!pkt || pos + 1 >= pkt_end)
 				goto error;
 			offset = gldns_read_uint16(pos) & 0x3FFF;
 			if (pkt + offset >= pkt_end)
@@ -204,7 +224,7 @@ error:
 	return NULL;
 }
 
-uint8_t *
+const uint8_t *
 _getdns_owner_if_or_as_decompressed(_getdns_rr_iter *i,
     uint8_t *ff_bytes, size_t *len)
 {
@@ -215,7 +235,7 @@ _getdns_owner_if_or_as_decompressed(_getdns_rr_iter *i,
 static _getdns_rdf_iter *
 rdf_iter_find_nxt(_getdns_rdf_iter *i)
 {
-	uint8_t *pos;
+	const uint8_t *pos;
 
 	assert(i);
 	assert(i->pos);
@@ -279,7 +299,7 @@ _getdns_rdf_iter_init(_getdns_rdf_iter *i, _getdns_rr_iter *rr)
 
 	i->end     = NULL;
 	/* rr_iter already done or in question section */
-	if (!rr->pos || rr->n < GLDNS_QDCOUNT(rr->pkt))
+	if (!rr->pos || _getdns_rr_iter_section(rr) == GLDNS_SECTION_QUESTION)
 		goto done;
 
 	i->pkt     = rr->pkt;
@@ -334,7 +354,7 @@ _getdns_rdf_iter_init_at(
 	return i;
 }       
 
-uint8_t *
+const uint8_t *
 _getdns_rdf_if_or_as_decompressed(
     _getdns_rdf_iter *i, uint8_t *ff_bytes, size_t *len)
 {
