@@ -116,8 +116,8 @@ struct getdns_upstream;
 #define TIMEOUT_FOREVER ((int64_t)-1)
 #define ASSERT_UNREACHABLE 0
 
-#define GETDNS_TRANSPORTS_MAX 4
-#define GETDNS_UPSTREAM_TRANSPORTS 3
+#define GETDNS_TRANSPORTS_MAX 3
+#define GETDNS_UPSTREAM_TRANSPORTS 2
 
 /** @}
  */
@@ -203,12 +203,17 @@ typedef struct getdns_network_req
 	/* request type */
 	uint16_t request_type;
 
-	/* request class */
-	uint16_t request_class;
-
 	/* dnssec status */
 	int dnssec_status;
 
+	/* tsig status:
+	 * GETDNS_DNSSEC_INDETERMINATE means "No TSIG processing"
+	 * GETDNS_DNSSEC_INSECURE      means "TSIG sent, validate reply"
+	 * GETDNS_DNSSEC_SECURE        means "Validated"
+	 * GETDNS_DNSSEC_BOGUS         means "Validation failed"
+	 */
+	int tsig_status;
+
 	/* For stub resolving */
 	struct getdns_upstream *upstream;
 	int                     fd;
@@ -223,10 +228,12 @@ typedef struct getdns_network_req
 	int                     edns_maximum_udp_payload_size;
 	uint16_t                max_udp_payload_size;
 
+	size_t                  keepalive_sent;
+
 	/* Network requests scheduled to write after me */
 	struct getdns_network_req *write_queue_tail;
 
-	/* Some fields to record info for return_call_debugging */
+	/* Some fields to record info for return_call_reporting */
 	uint64_t                    debug_start_time;
 	uint64_t                    debug_end_time;
 	size_t                      debug_tls_auth_status;
@@ -272,6 +279,13 @@ typedef struct getdns_dns_req {
 	uint8_t name[256];
 	size_t  name_len;
 
+	getdns_append_name_t append_name;
+	const uint8_t *suffix;
+	size_t  suffix_len;
+	int suffix_appended;
+
+	uint16_t request_class;
+
 	/* canceled flag */
 	int canceled;
 
@@ -289,7 +303,8 @@ typedef struct getdns_dns_req {
 	int edns_cookies;
 	int edns_client_subnet_private;
 	uint16_t tls_query_padding_blocksize;
-	int return_call_debugging;
+	int return_call_reporting;
+	int add_warning_for_bad_dns;
 
 	/* Internally used by return_validation_chain */
 	int dnssec_ok_checking_disabled;
@@ -379,5 +394,12 @@ getdns_return_t _getdns_network_req_add_upstream_option(getdns_network_req * req
 					     uint16_t code, uint16_t sz, const void* data);
 void _getdns_network_req_clear_upstream_options(getdns_network_req * req);
 
+/* Adds TSIG signature (if needed) and returns query length */
+size_t _getdns_network_req_add_tsig(getdns_network_req *req);
+
+void _getdns_network_validate_tsig(getdns_network_req *req);
+
+void _getdns_netreq_reinit(getdns_network_req *netreq);
+
 #endif
 /* types-internal.h */
