@@ -35,22 +35,22 @@
 
 const char *getdns_get_version(void)
 {
-	return "0.5.1";
+	return "0.9.0";
 }
 
 uint32_t getdns_get_version_number(void)
 {
-	return 0x00050100;
+	return 0x00090000;
 }
 
 const char *getdns_get_api_version(void)
 {
-	return "October 2015";
+	return "December 2015";
 }
 
 uint32_t getdns_get_api_version_number(void)
 {
-	return 0x07df0a00;
+	return 0x07df0c00;
 }
 
 /* version.c */
